-- Exercise 10: WHERE (numbers) ------------------------------------------------------
DROP TABLE plsql101_product;

CREATE TABLE plsql101_product ( 
     product_name     VARCHAR2(25), 
     product_price    NUMBER(4,2),
     quantity_on_hand NUMBER(5,0),
     last_stock_date  DATE
     )
;

INSERT INTO plsql101_product VALUES 
     ('Small Widget', 99, 1, '15-JAN-03');
INSERT INTO plsql101_product VALUES 
     ('Medium Wodget', 75, 1000, '15-JAN-02');
INSERT INTO plsql101_product VALUES 
     ('Chrome Phoobar', 50, 100, '15-JAN-03');
INSERT INTO plsql101_product VALUES 
     ('Round Chrome Snaphoo', 25, 10000, null);


SELECT * FROM plsql101_product
WHERE  quantity_on_hand = 1;

SELECT * FROM plsql101_product 
WHERE  quantity_on_hand < 500;

SELECT * FROM plsql101_product 
WHERE  quantity_on_hand < 1000;

SELECT * FROM plsql101_product 
WHERE  quantity_on_hand <= 1000;

SELECT * FROM plsql101_product 
WHERE  quantity_on_hand > 1000;

SELECT * FROM plsql101_product 
WHERE  quantity_on_hand >= 1000;

SELECT * FROM plsql101_product 
WHERE  quantity_on_hand > 1000;

SELECT * FROM plsql101_product 
WHERE  quantity_on_hand >= 1000;

SELECT * FROM plsql101_product
WHERE  product_price IN (50, 99);


-- Exercise 11: WHERE (text) ------------------------------------------------------
SELECT * FROM plsql101_product
WHERE  product_name = 'Small Widget';

SELECT * FROM plsql101_product
WHERE  product_name IN ('Small Widget', 'Round Chrome Snaphoo');

SELECT * FROM plsql101_product
WHERE  product_name LIKE 'Chrome%';

SELECT * FROM plsql101_product
WHERE  product_name LIKE '%Chrome%';

SELECT * FROM plsql101_product
WHERE  product_name LIKE '%W_d%';

-- Exercise 12: WHERE (dates) ------------------------------------------------------
SELECT * FROM plsql101_product
WHERE  last_stock_date = '15-JAN-03';

SELECT * FROM plsql101_product
WHERE  last_stock_date = '15-JAN-2003';

SELECT * FROM plsql101_product
WHERE  last_stock_date > '31-DEC-02';

SELECT * FROM plsql101_product
WHERE  last_stock_date BETWEEN '01-JAN-03' and '31-DEC-03';

SELECT * FROM plsql101_product
WHERE  last_stock_date NOT BETWEEN '01-JAN-03' and '31-DEC-03';

SELECT * FROM plsql101_product
WHERE  last_stock_date IS NULL;

SELECT * FROM plsql101_product
WHERE  last_stock_date IS NOT NULL;


-- Exercise 13: ORDER BY ------------------------------------------------------
SELECT * FROM plsql101_product ORDER BY product_price;
SELECT * FROM plsql101_product ORDER BY quantity_on_hand;
SELECT * FROM plsql101_product 
ORDER BY last_stock_date, product_name;

SELECT   last_stock_date, 
         product_name, 
         product_price, 
         quantity_on_hand
FROM     plsql101_product
ORDER BY last_stock_date, 
         product_name
;

SELECT * FROM plsql101_product ORDER BY product_price DESC;

SELECT product_name FROM plsql101_product ORDER BY quantity_on_hand;


-- Exercise 14: UNIQUE / DISTINCT ------------------------------------------------------
DROP TABLE plsql101_purchase;

CREATE TABLE plsql101_purchase ( 
     product_name  VARCHAR2(25), 
     quantity      NUMBER(4,2),
     purchase_date DATE, 
     salesperson   VARCHAR2(3)
     )
;

INSERT INTO plsql101_purchase VALUES 
     ('Small Widget', 1, '14-JUL-03', 'CA');
INSERT INTO plsql101_purchase VALUES 
     ('Medium Wodget', 75, '14-JUL-03', 'BB');
INSERT INTO plsql101_purchase VALUES 
     ('Chrome Phoobar', 2, '14-JUL-03', 'GA');
INSERT INTO plsql101_purchase VALUES 
     ('Small Widget', 8, '15-JUL-03', 'GA');
INSERT INTO plsql101_purchase VALUES 
     ('Medium Wodget', 20, '15-JUL-03', 'LB');
INSERT INTO plsql101_purchase VALUES 
     ('Chrome Phoobar', 2, '16-JUL-03', 'CA');
INSERT INTO plsql101_purchase VALUES 
     ('Round Snaphoo', 25, '16-JUL-03', 'LB');
INSERT INTO plsql101_purchase VALUES 
     ('Chrome Phoobar', 2, '17-JUL-03', 'BB');


SELECT UNIQUE product_name 
FROM          plsql101_purchase
ORDER BY      product_name;

SELECT DISTINCT product_name 
FROM            plsql101_purchase
ORDER BY        product_name;

SELECT UNIQUE product_name 
FROM          plsql101_purchase
ORDER BY      product_name;

SELECT DISTINCT salesperson
FROM            plsql101_purchase
WHERE           purchase_date BETWEEN '01-JUL-03' AND '15-JUL-03'
ORDER BY        salesperson;


-- Exercise 15: SELECTING FROM DUAL ------------------------------------------------------
SELECT 18*1.05 FROM plsql101_purchase;

DESC DUAL;
SELECT * FROM DUAL;

SELECT 18*1.05 FROM DUAL;

select 'ABC' from dual;

select 'A' || 'B' from dual;


-- Exercise 16: UPDATE ------------------------------------------------------
SELECT * FROM plsql101_purchase;

UPDATE plsql101_purchase
SET    product_name = 'Large Widget'
WHERE  product_name = 'Small Widget';

SELECT * FROM plsql101_purchase;


-- Exercise 17: DELETE ------------------------------------------------------
SELECT * FROM plsql101_purchase;

DELETE FROM plsql101_purchase
WHERE purchase_date > '15-JUL-03';

SELECT * FROM plsql101_purchase;


DELETE FROM plsql101_purchase
WHERE product_name = 'Large Widget';

SELECT * FROM plsql101_purchase;

-- delete from PLSQL101_PURCHASE;

TRUNCATE TABLE plsql101_purchase;

SELECT * FROM plsql101_purchase;


-- Exercise 18: TRANSACTION CONTROL ------------------------------------------------------
drop table PLSQL101_PURCHASE;

CREATE TABLE plsql101_purchase ( 
     product_name  VARCHAR2(25), 
     quantity      NUMBER(4,2),
     purchase_date DATE, 
     salesperson   VARCHAR2(3)
     )
;


INSERT INTO plsql101_purchase VALUES 
     ('Small Widget', 1, '14-JUL-03', 'CA');
INSERT INTO plsql101_purchase VALUES 
     ('Medium Wodget', 75, '14-JUL-03', 'BB');


SELECT * FROM plsql101_purchase;

ROLLBACK;

SELECT * FROM plsql101_purchase;


INSERT INTO plsql101_purchase VALUES 
     ('Small Widget', 1, '14-JUL-03', 'CA');
SAVEPOINT a;
INSERT INTO plsql101_purchase VALUES 
     ('Medium Wodget', 75, '14-JUL-03', 'BB');
SAVEPOINT sp_2;
INSERT INTO plsql101_purchase VALUES 
     ('Chrome Phoobar', 2, '14-JUL-03', 'GA');
SAVEPOINT third;
INSERT INTO plsql101_purchase VALUES 
     ('Small Widget', 8, '15-JUL-03', 'GA');
SAVEPOINT final_sp;
INSERT INTO plsql101_purchase VALUES 
     ('Medium Wodget', 20, '15-JUL-03', 'LB');
SELECT * FROM plsql101_purchase;
ROLLBACK TO final_sp;
SELECT * FROM plsql101_purchase;
ROLLBACK TO third;
SELECT * FROM plsql101_purchase;
ROLLBACK TO sp_2;
SELECT * FROM plsql101_purchase;
ROLLBACK TO a;
SELECT * FROM plsql101_purchase;
ROLLBACK;
SELECT * FROM plsql101_purchase;


INSERT INTO plsql101_purchase VALUES 
     ('Small Widget', 1, '14-JUL-03', 'CA');
SAVEPOINT A;
INSERT INTO plsql101_purchase VALUES 
     ('Medium Wodget', 75, '14-JUL-03', 'BB');
SAVEPOINT B;
INSERT INTO plsql101_purchase VALUES 
     ('Chrome Phoobar', 2, '14-JUL-03', 'GA');
SAVEPOINT C;
INSERT INTO plsql101_purchase VALUES 
     ('Small Widget', 8, '15-JUL-03', 'GA');
SAVEPOINT D;
INSERT INTO plsql101_purchase VALUES 
     ('Medium Wodget', 20, '15-JUL-03', 'LB');

COMMIT;

ROLLBACK TO D;

SELECT * FROM plsql101_purchase;


-- Exercise 19: DATA CONSISTENCY ------------------------------------------------------
-- Create a pair of scripts to be executed by two SQL*Plus sessions 
-- to show the back-and-forth of making changes visible to other users.

-- Commands for first SQL*Plus window
select * from PLSQL101_PURCHASE;











insert into PLSQL101_PURCHASE values (
     'Round Snaphoo', 5, '16-JUL-03', 'CA');











commit;

-- ____________________________________________________________
-- Commands for second SQL*Plus window










select * from PLSQL101_PURCHASE;






select * from PLSQL101_PURCHASE;






select * from PLSQL101_PURCHASE;
____________________________________________________________


select * from PRODUCT
where  COLOR = 'Red'
       or
       COLOR = 'Green'
       or
       COLOR = 'White'
;

select * from PRODUCT
where  COLOR in ('Red', 'Green', 'White')
;
