-- ======================= BEGINNING OF CHAPTER 6 =======================
/* Catch-up if starting in this chapter without doing previous chapters' exercises


-- ======================== PERSON ==================================
DROP TABLE plsql101_person;
CREATE TABLE plsql101_person (
     person_code VARCHAR2(3),
     first_name  VARCHAR2(15),
     last_name   VARCHAR2(20),
     hire_date   DATE
     )
;

INSERT INTO plsql101_person VALUES
     ('CA', 'Charlene', 'Atlas', '01-FEB-02');
INSERT INTO plsql101_person VALUES
     ('GA', 'Gary', 'Anderson', '15-FEB-02');
INSERT INTO plsql101_person VALUES
     ('BB', 'Bobby', 'Barkenhagen', '28-FEB-02');
INSERT INTO plsql101_person VALUES
     ('LB', 'Laren', 'Baxter', '01-MAR-02');



-- ================= PRODUCT ========================

DROP TABLE plsql101_product;
CREATE TABLE plsql101_product (
     product_name     VARCHAR2(25),
     product_price    NUMBER(4,2),
     quantity_on_hand NUMBER(5,0),
     last_stock_date  DATE
     )
;

INSERT INTO plsql101_product VALUES
     ('Small Widget', 99, 1, '15-JAN-03');
INSERT INTO plsql101_product VALUES
     ('Medium Wodget', 75, 1000, '15-JAN-02');
INSERT INTO plsql101_product VALUES
     ('Chrome Phoobar', 50, 100, '15-JAN-03');
INSERT INTO plsql101_product VALUES
     ('Round Chrome Snaphoo', 25, 10000, null);
INSERT INTO plsql101_product VALUES
     ('Extra Huge Mega Phoobar +',9.95,1234,'15-JAN-04');
INSERT INTO plsql101_product VALUES ('Square Zinculator', 
     45, 1, TO_DATE('December 31, 2002, 11:30 P.M.',
                       'Month dd, YYYY, HH:MI P.M.')
     )
;


-- =================== PURCHASE ========================
DROP TABLE plsql101_purchase;
CREATE TABLE plsql101_purchase ( 
     product_name  VARCHAR2(25), 
     salesperson   VARCHAR2(3),
     purchase_date DATE, 
     quantity      NUMBER(4,2)
     )
;

INSERT INTO plsql101_purchase VALUES 
     ('Small Widget', 'CA', '14-JUL-03', 1);
INSERT INTO plsql101_purchase VALUES 
     ('Medium Wodget', 'BB', '14-JUL-03', 75);
INSERT INTO plsql101_purchase VALUES 
     ('Chrome Phoobar', 'GA', '14-JUL-03', 2);
INSERT INTO plsql101_purchase VALUES 
     ('Small Widget', 'GA', '15-JUL-03', 8);
INSERT INTO plsql101_purchase VALUES 
     ('Medium Wodget', 'LB', '15-JUL-03', 20);
INSERT INTO plsql101_purchase VALUES 
     ('Round Snaphoo', 'CA', '16-JUL-03', 5);

-- Note: The "Round Snaphoo" records in PRODUCT and PURCHASE are
-- out of sync in terms of product name.  Good opportunity to use this
-- later to enforce the importance of referential integrity.


-- =========================== OLD_ITEM ===============================
DROP TABLE plsql101_old_item;
CREATE TABLE plsql101_old_item (
     item_id   CHAR(20),
     item_desc CHAR(25)
     )
;

INSERT INTO plsql101_old_item VALUES
     ('LA-101', 'Can, Small');
INSERT INTO plsql101_old_item VALUES
     ('LA-102', 'Can, Large');
INSERT INTO plsql101_old_item VALUES
     ('LA-103', 'Bottle, Small');
INSERT INTO plsql101_old_item VALUES
     ('LA-104', 'Bottle, Large');
INSERT INTO plsql101_old_item VALUES
     ('NY-101', 'Box, Small');
INSERT INTO plsql101_old_item VALUES
     ('NY-102', 'Box, Large');
INSERT INTO plsql101_old_item VALUES
     ('NY-103', 'Shipping Carton, Small');
INSERT INTO plsql101_old_item VALUES
     ('NY-104', 'Shipping Carton, Large');



*/

-- Exercise 54: Indexes
/*
drop INDEX t1_index_c1; 

timing start
update t1 set c3='01-JUN-99' where
c1 > 20000;
timing stop

timing start
update t1 set c3='01-JUN-99' where
c1 > 20000;
timing stop

CREATE INDEX t1_index_c1 ON t1(c1); 

timing start
update t1 set c3='01-JUN-99' where
c1 > 20000;
timing stop

timing start
update t1 set c3='01-JUN-99' where
c1 > 20000;
timing stop

-- Start the deletion test
timing start
delete from t1 where c1=20000;
timing stop

timing start
delete from t1 where c1=19999;
timing stop

drop INDEX t1_index_c1; 

timing start
delete from t1 where c1=19998;
timing stop

timing start
delete from t1 where c1=19997;
timing stop


*/

create index PLSQL101_PERSON_CODE_INDEX 
on PLSQL101_PERSON(PERSON_CODE);

create index PLSQL101_PERSON_NAME_INDEX 
on PLSQL101_PERSON(LAST_NAME, FIRST_NAME);

create index PLSQL101_PURCHASE_PRODUCT
on PLSQL101_PURCHASE(PRODUCT_NAME);

create index PLSQL101_PURCHASE_SALESPERSON
on PLSQL101_PURCHASE(SALESPERSON);


-- NOT NULL CONSTRAINT
ALTER TABLE plsql101_person MODIFY (first_name NOT NULL);
ALTER TABLE plsql101_person MODIFY (last_name NOT NULL);

INSERT INTO plsql101_person VALUES (
     'XL', 'Xaviera', NULL, '15-NOV-03'
     )
;



-- UNIQUE CONSTRAINT
ALTER TABLE plsql101_person 
ADD CONSTRAINT plsql101_person_unique UNIQUE (
     first_name,
     last_name,
     hire_date
     )
;

INSERT INTO plsql101_person VALUES (
     'LN', 'Linda', 'Norton', '01-JUN-03');

INSERT INTO plsql101_person VALUES (
     'NL', 'Linda', 'Norton', '01-JUN-03');




-- CHECK CONSTRAINTS

ALTER TABLE plsql101_purchase ADD ( 
     CONSTRAINT reasonable_date CHECK(
          purchase_date IS NOT NULL
          AND
          TO_CHAR(purchase_date, 'YYYY-MM-DD') >= '2000-06-30'
          )
     )
;

INSERT INTO plsql101_purchase VALUES (
     'Small Widget', 'GA', '28-FEB-00', 10);

/* 
-- Following does not check for nulls in LAST_STOCK_DATE, because it's possible 
-- a product could be added to the list before it is received.
ALTER TABLE plsql101_product ADD ( 
     CONSTRAINT valid_price CHECK(
          product_price IS NOT NULL 
          AND
          product_price BETWEEN 0 AND 10000
          )
     )
;

ALTER TABLE plsql101_product ADD ( 
     CONSTRAINT positive_quantity CHECK(
          quantity_on_hand IS NOT NULL
          AND
          quantity_on_hand >=0
          )
     )
;
*/

/* ALTER TABLE plsql101_product ADD ( 
     CONSTRAINT reasonable_date CHECK(
          TO_CHAR(last_stock_date, 'YYYY-MM-DD') >= '2001-12-31'
          )
     )
;

INSERT INTO plsql101_product VALUES (
     'Small Widget', NULL, 1, '28-FEB-02');

INSERT INTO plsql101_product VALUES (
     'Small Widget', 10001, 1, '28-FEB-02');

INSERT INTO plsql101_product VALUES (
     'Small Widget', 1.95, -5, '28-FEB-02');

INSERT INTO plsql101_product VALUES (
     'Small Widget', 1.95, 1, '30-DEC-2001');


*/

-- Exercise: Adding constraint that checks values and prohobits nulls
ALTER TABLE plsql101_purchase ADD ( 
     CONSTRAINT reasonable_date CHECK(
          purchase_date IS NOT NULL
          AND
          TO_CHAR(purchase_date, 'YYYY-MM-DD') >= '2000-06-30'
          )
     )
;

INSERT INTO plsql101_purchase VALUES (
     'Small Widget', 10, '28-FEB-00', 'GA');




-- Exercise: Adding a check constraint that allows null values
ALTER TABLE plsql101_product ADD ( 
     CONSTRAINT reasonable_stock_date CHECK(
          TO_CHAR(last_stock_date, 'YYYY-MM-DD') >= '2001-12-31'
          )
     )
;

INSERT INTO plsql101_product VALUES (
     'Anodized Framifier', 49, 5, NULL)
;

INSERT INTO plsql101_product VALUES (
     'Spring-Loaded Pit Puller', 49, 5, '30-DEC-01')
;


-- Exercise: Disabling and re-enabling constraints
INSERT INTO plsql101_product VALUES (
     'Red Snaphoo', 1.95, 10, '30-DEC-01')
;

ALTER TABLE plsql101_product DISABLE CONSTRAINT reasonable_stock_date;

INSERT INTO plsql101_product VALUES (
     'Red Snaphoo', 1.95, 10, '30-DEC-01')
;

ALTER TABLE plsql101_product ENABLE CONSTRAINT reasonable_stock_date;

UPDATE plsql101_product 
SET    last_stock_date = '31-DEC-01' 
WHERE  last_stock_date = '30-DEC-01';

ALTER TABLE plsql101_product ENABLE CONSTRAINT reasonable_stock_date;

-- delete from plsql101_product where product_name='Red Snaphoo';


-- Exercise: Changing a column's NULL status
ALTER TABLE plsql101_person MODIFY (first_name NULL);
ALTER TABLE plsql101_person MODIFY (last_name NULL);



-- Exercise: Dropping a constraint
INSERT INTO plsql101_product VALUES (
     'Blue Snaphoo', 1.95, 10, '30-DEC-01')
;

ALTER TABLE plsql101_product DROP CONSTRAINT reasonable_stock_date;

INSERT INTO plsql101_product VALUES (
     'Blue Snaphoo', 1.95, 10, '30-DEC-01')
;



-- Exercise: Adding a primary key to an existing table
ALTER TABLE plsql101_product
ADD PRIMARY KEY (product_name);

ALTER TABLE plsql101_person
ADD PRIMARY KEY (person_code);

ALTER TABLE plsql101_purchase
ADD PRIMARY KEY (product_name,
                 salesperson,
                 purchase_date
                 )
;




/*
-- Example: Specifying a primary key when a table is created
CREATE TABLE plsql101_product2 (
     product_name     VARCHAR2(25) PRIMARY KEY,
     product_price    NUMBER(4,2),
     quantity_on_hand NUMBER(5,0),
     last_stock_date  DATE
     )
;
*/


-- Exercise: Manually locating a child record with no matching parent record
SELECT product_name
FROM   plsql101_product
ORDER BY product_name;

SELECT DISTINCT product_name
FROM   plsql101_purchase
ORDER BY product_name;


-- Exercise: Attempting to create a foreign-key constraint 
--           with non-matching child values
ALTER TABLE plsql101_purchase
ADD CONSTRAINT plsql101_purchase_fk_product
          FOREIGN KEY (product_name) 
          REFERENCES plsql101_product;

-- Exercise: Fixing child values & creating the FK constraint
UPDATE plsql101_purchase
SET    product_name = 'Round Chrome Snaphoo'
WHERE  product_name = 'Round Snaphoo';

ALTER TABLE plsql101_purchase
ADD CONSTRAINT plsql101_purchase_fk_product
          FOREIGN KEY (product_name) 
          REFERENCES plsql101_product;


-- Exercise: Attempting to enter a child record without a matching 
-- parent record
INSERT INTO plsql101_purchase VALUES (
     'Small Widgee', 'CA', '17-JUL-03', 1)
;

-- Corrected version
INSERT INTO plsql101_purchase VALUES (
     'Small Widget', 'CA', '17-JUL-03', 1)
;



-- Readers are supposed to figure out how to build the following command
ALTER TABLE plsql101_purchase
ADD CONSTRAINT plsql101_purchase_fk_person
          FOREIGN KEY (salesperson)
          REFERENCES plsql101_person;


-- Exercise: Writing a SELECT statement that joins records from two tables
SELECT plsql101_purchase.product_name,
       plsql101_person.last_name,
       plsql101_person.first_name,
       plsql101_purchase.quantity
FROM   plsql101_purchase,
       plsql101_person
WHERE  plsql101_person.person_code = plsql101_purchase.salesperson
;


-- Exercise: Creating a join without a WHERE clause
SELECT plsql101_purchase.product_name,
       plsql101_person.last_name,
       plsql101_person.first_name,
       plsql101_purchase.quantity
FROM   plsql101_purchase,
       plsql101_person
;

-- Exercise: Writing a SELECT statement that joins records from three tables
SELECT plsql101_purchase.product_name,
       plsql101_product.product_price,
       plsql101_purchase.quantity,
       plsql101_person.last_name
FROM   plsql101_product,
       plsql101_person,
       plsql101_purchase
WHERE  plsql101_product.product_name = plsql101_purchase.product_name
       and
       plsql101_person.person_code = plsql101_purchase.salesperson
;


-- Readers are supposed to figure out how to build the following command
SELECT plsql101_purchase.purchase_date,
       plsql101_purchase.quantity,
       plsql101_product.last_stock_date,
       plsql101_person.last_name
FROM   plsql101_product,
       plsql101_person,
       plsql101_purchase
WHERE  plsql101_product.product_name = plsql101_purchase.product_name
       and
       plsql101_person.person_code = plsql101_purchase.salesperson
;


-- Exercise: Outer joins
SELECT product_name FROM plsql101_product ORDER BY product_name;

SELECT prod.product_name,
       prod.product_price,
       purc.purchase_date,
       purc.quantity
FROM   plsql101_product  prod,
       plsql101_purchase purc
WHERE  prod.product_name = purc.product_name
ORDER BY prod.product_name;

SELECT prod.product_name,
       prod.product_price,
       purc.purchase_date,
       purc.quantity
FROM   plsql101_product  prod,
       plsql101_purchase purc
WHERE  prod.product_name = purc.product_name (+)
ORDER BY prod.product_name;


-- Exercise: Join Operators
CREATE TABLE plsql101_purchase_archive ( 
     product_name  VARCHAR2(25), 
     salesperson   VARCHAR2(3),
     purchase_date DATE, 
     quantity      NUMBER(4,2)
     )
;

INSERT INTO plsql101_purchase_archive VALUES 
     ('Round Snaphoo', 'BB', '21-JUN-01', 10);
INSERT INTO plsql101_purchase_archive VALUES 
     ('Large Harflinger', 'GA', '22-JUN-01', 50);
INSERT INTO plsql101_purchase_archive VALUES 
     ('Medium Wodget', 'LB', '23-JUN-01', 20);
INSERT INTO plsql101_purchase_archive VALUES 
     ('Small Widget', 'ZZ', '24-JUN-02', 80);
INSERT INTO plsql101_purchase_archive VALUES 
     ('Chrome Phoobar', 'CA', '25-JUN-02', 2);
INSERT INTO plsql101_purchase_archive VALUES 
     ('Small Widget', 'JT', '26-JUN-02', 50);

SELECT product_name FROM plsql101_purchase 
ORDER BY product_name;
SELECT product_name FROM plsql101_purchase_archive 
ORDER BY product_name;

SELECT product_name FROM plsql101_purchase
UNION
SELECT product_name FROM plsql101_purchase_archive
ORDER BY product_name;

SELECT product_name FROM plsql101_purchase
UNION ALL
SELECT product_name FROM plsql101_purchase_archive
ORDER BY product_name;

SELECT product_name FROM plsql101_purchase
INTERSECT
SELECT product_name FROM plsql101_purchase_archive
ORDER BY product_name;

SELECT product_name FROM plsql101_purchase
MINUS
SELECT product_name FROM plsql101_purchase_archive
ORDER BY product_name;


-- Exercise: Single-Row Subquery
SELECT * 
FROM   plsql101_product
WHERE  last_stock_date = (
       SELECT last_stock_date
       FROM   plsql101_product
       WHERE  product_name = 'Small Widget'
       )
;


-- Readers are supposed to figure out how to build the following command
SELECT * 
FROM   plsql101_product
WHERE  product_price = (
       SELECT product_price
       FROM   plsql101_product
       WHERE  product_name = 'Red Snaphoo'
       )
;

-- Exercise: How to have the subquery select from a different table
-- than the parent statement
SELECT * FROM plsql101_purchase 
WHERE  salesperson = (
       SELECT person_code
       FROM   plsql101_person
       WHERE  first_name = 'Gary' AND last_name = 'Anderson'
       )
;

-- Exercise: Show products that are more expensive than average
SELECT * 
FROM   plsql101_product
WHERE  product_price > (
       SELECT AVG(product_price)
       FROM   plsql101_product
       )
;


-- Multi-row subqueries
-- Exercise: Show products that aren't selling
SELECT * 
FROM  plsql101_purchase
ORDER BY product_name;

SELECT * 
FROM   plsql101_product
WHERE  product_name NOT IN (
       SELECT DISTINCT product_name
       FROM   plsql101_purchase
       )
ORDER BY product_name
;


-- Exercise: Update price of products that aren't selling
SELECT * FROM  plsql101_product;

UPDATE plsql101_product
SET    product_price = product_price * .9
WHERE  product_name NOT IN (
       SELECT DISTINCT product_name
       FROM   plsql101_purchase
       )
;

SELECT * FROM  plsql101_product;



-- Exercise: Multi-column subquery
SELECT * 
FROM   plsql101_purchase
ORDER BY product_name, purchase_date;

SELECT *
FROM   plsql101_purchase
WHERE  (product_name, purchase_date)
IN     (SELECT product_name, MAX(purchase_date)
        FROM   plsql101_purchase
        GROUP BY product_name
       )
;
