-- ======================= BEGINNING OF CHAPTER 7 =======================
/* Catch-up if starting in this chapter without doing 
   previous chapters' exercises


-- ======================== PERSON ==================================
DROP TABLE plsql101_purchase;
DROP TABLE plsql101_product;
DROP TABLE plsql101_person;
DROP TABLE plsql101_old_item;
DROP TABLE plsql101_purchase_archive;

CREATE TABLE plsql101_person (
     person_code VARCHAR2(3) PRIMARY KEY,
     first_name  VARCHAR2(15),
     last_name   VARCHAR2(20),
     hire_date   DATE
     )
;

CREATE INDEX plsql101_person_name_index 
ON plsql101_person(last_name, first_name);

ALTER TABLE plsql101_person 
ADD CONSTRAINT plsql101_person_unique UNIQUE (
     first_name,
     last_name,
     hire_date
     )
;

INSERT INTO plsql101_person VALUES
     ('CA', 'Charlene', 'Atlas', '01-FEB-02');
INSERT INTO plsql101_person VALUES
     ('GA', 'Gary', 'Anderson', '15-FEB-02');
INSERT INTO plsql101_person VALUES
     ('BB', 'Bobby', 'Barkenhagen', '28-FEB-02');
INSERT INTO plsql101_person VALUES
     ('LB', 'Laren', 'Baxter', '01-MAR-02');
INSERT INTO plsql101_person VALUES (
     'LN', 'Linda', 'Norton', '01-JUN-03');

CREATE TABLE plsql101_product (
     product_name     VARCHAR2(25) PRIMARY KEY,
     product_price    NUMBER(4,2),
     quantity_on_hand NUMBER(5,0),
     last_stock_date  DATE
     )
;

ALTER TABLE plsql101_product ADD CONSTRAINT positive_quantity CHECK(
     quantity_on_hand IS NOT NULL
     AND
     quantity_on_hand >=0
     )
;

INSERT INTO plsql101_product VALUES
     ('Small Widget', 99, 1, '15-JAN-03');
INSERT INTO plsql101_product VALUES
     ('Medium Wodget', 75, 1000, '15-JAN-02');
INSERT INTO plsql101_product VALUES
     ('Chrome Phoobar', 50, 100, '15-JAN-03');
INSERT INTO plsql101_product VALUES
     ('Round Chrome Snaphoo', 25, 10000, null);
INSERT INTO plsql101_product VALUES
     ('Extra Huge Mega Phoobar +',9.95,1234,'15-JAN-04');
INSERT INTO plsql101_product VALUES ('Square Zinculator', 
     45, 1, TO_DATE('December 31, 2002, 11:30 P.M.',
                       'Month dd, YYYY, HH:MI P.M.')
     )
;
INSERT INTO plsql101_product VALUES (
     'Anodized Framifier', 49, 5, NULL);
INSERT INTO plsql101_product VALUES (
     'Red Snaphoo', 1.95, 10, '31-DEC-01');
INSERT INTO plsql101_product VALUES (
     'Blue Snaphoo', 1.95, 10, '30-DEC-01')
;


CREATE TABLE plsql101_purchase ( 
     product_name  VARCHAR2(25), 
     salesperson   VARCHAR2(3),
     purchase_date DATE, 
     quantity      NUMBER(4,2)
     )
;

ALTER TABLE plsql101_purchase
ADD PRIMARY KEY (product_name,
                 salesperson,
                 purchase_date
                 )
;

ALTER TABLE plsql101_purchase ADD CONSTRAINT reasonable_date CHECK(
     purchase_date IS NOT NULL
     AND
     TO_CHAR(purchase_date, 'YYYY-MM-DD') >= '2000-06-30'
     )
;

ALTER TABLE plsql101_purchase
     ADD CONSTRAINT plsql101_purchase_fk_product FOREIGN KEY 
     (product_name) REFERENCES plsql101_product;

ALTER TABLE plsql101_purchase
     ADD CONSTRAINT plsql101_purchase_fk_person FOREIGN KEY 
     (salesperson) REFERENCES plsql101_person;

CREATE INDEX plsql101_purchase_product
ON plsql101_purchase(product_name);

CREATE INDEX plsql101_purchase_salesperson
ON plsql101_purchase(salesperson);

INSERT INTO plsql101_purchase VALUES 
     ('Small Widget', 'CA', '14-JUL-03', 1);
INSERT INTO plsql101_purchase VALUES 
     ('Medium Wodget', 'BB', '14-JUL-03', 75);
INSERT INTO plsql101_purchase VALUES 
     ('Chrome Phoobar', 'GA', '14-JUL-03', 2);
INSERT INTO plsql101_purchase VALUES 
     ('Small Widget', 'GA', '15-JUL-03', 8);
INSERT INTO plsql101_purchase VALUES 
     ('Medium Wodget', 'LB', '15-JUL-03', 20);
INSERT INTO plsql101_purchase VALUES 
     ('Round Chrome Snaphoo', 'CA', '16-JUL-03', 5);
INSERT INTO plsql101_purchase VALUES (
     'Small Widget', 'CA', '17-JUL-03', 1)
;

UPDATE plsql101_product
SET    product_price = product_price * .9
WHERE  product_name NOT IN (
       SELECT DISTINCT product_name
       FROM   plsql101_purchase
       )
;

CREATE TABLE plsql101_old_item (
     item_id   CHAR(20),
     item_desc CHAR(25)
     )
;

INSERT INTO plsql101_old_item VALUES
     ('LA-101', 'Can, Small');
INSERT INTO plsql101_old_item VALUES
     ('LA-102', 'Can, Large');
INSERT INTO plsql101_old_item VALUES
     ('LA-103', 'Bottle, Small');
INSERT INTO plsql101_old_item VALUES
     ('LA-104', 'Bottle, Large');
INSERT INTO plsql101_old_item VALUES
     ('NY-101', 'Box, Small');
INSERT INTO plsql101_old_item VALUES
     ('NY-102', 'Box, Large');
INSERT INTO plsql101_old_item VALUES
     ('NY-103', 'Shipping Carton, Small');
INSERT INTO plsql101_old_item VALUES
     ('NY-104', 'Shipping Carton, Large');


CREATE TABLE plsql101_purchase_archive ( 
     product_name  VARCHAR2(25), 
     salesperson   VARCHAR2(3),
     purchase_date DATE, 
     quantity      NUMBER(4,2)
     )
;

INSERT INTO plsql101_purchase_archive VALUES 
     ('Round Snaphoo', 'BB', '21-JUN-01', 10);
INSERT INTO plsql101_purchase_archive VALUES 
     ('Large Harflinger', 'GA', '22-JUN-01', 50);
INSERT INTO plsql101_purchase_archive VALUES 
     ('Medium Wodget', 'LB', '23-JUN-01', 20);
INSERT INTO plsql101_purchase_archive VALUES 
     ('Small Widget', 'ZZ', '24-JUN-02', 80);
INSERT INTO plsql101_purchase_archive VALUES 
     ('Chrome Phoobar', 'CA', '25-JUN-02', 2);
INSERT INTO plsql101_purchase_archive VALUES 
     ('Small Widget', 'JT', '26-JUN-02', 50);

*/


-- ==================== BEGIN ACTIVITIES FOR THIS CHAPTER ===============
-- Exercise: Copy data from one or more tables to another
CREATE TABLE plsql101_purchase_log (
     purchase_date    DATE, 
     product_name     VARCHAR2(25),
     product_price    NUMBER(4,2),
     quantity         NUMBER(4,2),
     sales_first_name VARCHAR2(15),
     sales_last_name  VARCHAR2(20)
     )
;

INSERT INTO plsql101_purchase_log (
     SELECT purc.purchase_date,
            prod.product_name,
            prod.product_price,
            purc.quantity,
            pers.first_name,
            pers.last_name
     FROM   plsql101_product  prod,
            plsql101_person   pers,
            plsql101_purchase purc
     WHERE  prod.product_name = purc.product_name
            AND
            pers.person_code = purc.salesperson
     )
;

SELECT * FROM plsql101_purchase_log;



-- Exercise: Create a table from one or more other tables
CREATE TABLE plsql101_purchase_log2 AS 
     SELECT purc.purchase_date,
            prod.product_name,
            prod.product_price,
            purc.quantity,
            pers.first_name,
            pers.last_name
     FROM   plsql101_product  prod,
            plsql101_person   pers,
            plsql101_purchase purc
     WHERE  prod.product_name = purc.product_name
            AND
            pers.person_code = purc.salesperson
;

SELECT * FROM plsql101_purchase_log2;


-- Exercise: Rename a table
RENAME plsql101_purchase_log2 TO plsql101_log;


-- Exercise: Add columns
DESC plsql101_log

ALTER TABLE plsql101_log
ADD data_load_date VARCHAR2(8) NULL;

DESC plsql101_log


-- Exercise: Change datatype of an existing column
DESC plsql101_log

ALTER TABLE plsql101_log
MODIFY data_load_date DATE;

DESC plsql101_log


-- Exercise: Change null status of an existing column
UPDATE plsql101_log SET data_load_date = '15-DEC-2003';

DESC plsql101_log

ALTER TABLE plsql101_log MODIFY data_load_date NOT NULL;

DESC plsql101_log


-- Exercise: Creating a simple filtering view
SELECT * FROM plsql101_purchase;

CREATE OR REPLACE VIEW plsql101_sales_by_atlas_v AS
SELECT * 
FROM   plsql101_purchase
WHERE  salesperson = 'CA'
;

SELECT * FROM plsql101_sales_by_atlas_v;


-- Exercise: Creating a view that joins tables
CREATE OR REPLACE VIEW plsql101_sales_per_person_v AS
SELECT pers.first_name || ' ' || pers.last_name SALESPERSON,
       purc.product_name, 
       purc.purchase_date,
       purc.quantity
FROM   plsql101_person   pers,
       plsql101_purchase purc
WHERE  pers.person_code = purc.salesperson (+)
;

SELECT * FROM plsql101_sales_per_person_v
ORDER BY salesperson, product_name, purchase_date;


-- Exercise: Dropping a view
DROP VIEW plsql101_sales_per_person_v;


-- Exercise: Top 'N' analysis
CREATE OR REPLACE VIEW plsql101_overstocked_items AS
SELECT product_name, quantity_on_hand
FROM   plsql101_product
WHERE  rownum <= 3
ORDER BY quantity_on_hand
;

SELECT * FROM plsql101_overstocked_items;



-- Exercise: Creating a sequence
DROP SEQUENCE plsql101_test_seq;
CREATE SEQUENCE plsql101_test_seq;

SELECT plsql101_test_seq.nextval FROM DUAL;
SELECT plsql101_test_seq.nextval FROM DUAL;
SELECT plsql101_test_seq.nextval FROM DUAL;


-- Exercise: Using a sequence to populate a table's column
CREATE TABLE plsql101_test (
     record_id   NUMBER(18,0),
     record_text VARCHAR2(10)
);



INSERT INTO plsql101_test VALUES (
     plsql101_test_seq.nextval,
     'Record A'
);

INSERT INTO plsql101_test VALUES (
     plsql101_test_seq.nextval,
     'Record B'
);

SELECT * FROM plsql101_test;


-- Exercise: Altering an existing sequence
ALTER SEQUENCE plsql101_test_seq
MAXVALUE 10
;

SELECT plsql101_test_seq.nextval FROM DUAL;
SELECT plsql101_test_seq.nextval FROM DUAL;
SELECT plsql101_test_seq.nextval FROM DUAL;
SELECT plsql101_test_seq.nextval FROM DUAL;
SELECT plsql101_test_seq.nextval FROM DUAL;
SELECT plsql101_test_seq.nextval FROM DUAL;


-- Exercise: Creating a synonym for a table
SELECT * FROM prod;

CREATE SYNONYM prod FOR plsql101_product;

SELECT * FROM prod;


-- Exercise: Creating a public synonym
CREATE PUBLIC SYNONYM plsql101_product FOR plsql101_product;


-- Exercise: Dropping a synonym
DROP SYNONYM prod;


-- Exercise: Dropping a public synonym
DROP PUBLIC SYNONYM plsql101_product;


-- Exercise: Seeing the objects in the Oracle data dictionary
SELECT table_name, SUBSTR(comments, 1, 45)
FROM dict
WHERE SUBSTR(comments, 1, 7) <> 'Synonym'
;


-- Exercise: Getting a list of your own tables
SELECT table_name FROM user_tables;


-- Exercise: Getting a list of your own views
SELECT view_name FROM user_views;


